
local DBL_EPSILON = 2.2204460492503131e-16

-- http://www.lighthouse3d.com/tutorials/maths/ray-triangle-intersection/
-- rayPos: Vec3, rayDir: Vec3
-- triangle: { Vec3, Vec3, Vec3 }
function intersectRayTri(rayPos, rayDir, triangle)
	local e1 = triangle[2] - triangle[1]
	local e2 = triangle[3] - triangle[1]
	local h  = rayDir:cross(e2)
	local a  = h:dot(e1)

	-- if a is too close to 0, ray does not intersect triangle
	if math.abs(a) <= DBL_EPSILON then
		return false
	end

	local f = 1 / a
	local s = rayPos - triangle[1]
	local u = s:dot(h) * f

	-- ray does not intersect triangle
	if u < 0 or u > 1 then
		return false
	end

	local q = s:cross(e1)
	local v = rayDir:dot(q) * f

	-- ray does not intersect triangle
	if v < 0 or u + v > 1 then
		return false
	end

	-- at this stage we can compute t to find out where
	-- the intersection point is on the line
	local t = q:dot(e2) * f

	-- return position of intersection and distance from ray origin
	if t >= DBL_EPSILON then
		return rayPos + rayDir * t, t
	end

	-- ray does not intersect triangle
	return false
end
