﻿
if not love.isVersionCompatible("11.0") then
	print("Your version of LÖVE is incompatible with this program!")
	os.exit(1)
end

require("3d")
require("object")

local winW, winH = love.graphics.getDimensions()

local ground = Ground(20)
local player = Player(Vec3(0, 0, 5))

local cube1 = Cube(Vec3(-1, 0.5, 2))
local cube2 = Cube(Vec3(1, 0.5, 2))

local portal1 = Portal(Vec3(0, 1, -2))
portal1.grabbable = true
local portal2 = Portal(Vec3(4, 1, -2), Vec3(0, -math.pi/2, 0))
portal1.dest, portal2.dest = portal2, portal1

local scene = { ground, player, cube1, cube2, portal1, portal2 }


local paused = false
love.mouse.setRelativeMode(not paused)


local holding = nil

local t0 = love.timer.getTime()

function love.update(dt)
	local dx, dz = 0, 0
	if love.keyboard.isDown("w") then dz = dz - 1 end
	if love.keyboard.isDown("s") then dz = dz + 1 end
	if love.keyboard.isDown("a") then dx = dx - 1 end
	if love.keyboard.isDown("d") then dx = dx + 1 end
	if dx ~= 0 or dz ~= 0 then
        local speed = 4
        player.pos = player.pos + Mat4.fromAngleAxis(player.orient[2], Vec3.e_y) * Vec3(dx, 0, dz):normalize()*speed*dt
	end
	
	if holding then
		holding.obj:setTransform(player:getCamera():localToGlobal() * holding.viewModel)
	end
	
	local t = love.timer.getTime() - t0
	--cube.orient[2] = t*0.5
end

local function principalAngle(a)
	if a <= -math.pi or a > math.pi then
		return (a + math.pi) % (2*math.pi) - math.pi
	else
		return a
	end
end

function love.mousemoved(x,y, dx,dy)
	if not paused then
		local speed = 4.0e-3
	    player.orient[2] = principalAngle(player.orient[2] - dx * speed)
		player.orient[1] = math.max(math.min(player.orient[1] - dy * speed, math.pi/2), -1*math.pi/2)
	end
end

function love.mousepressed(x, y, button)
	if not paused and button == 1 and not holding then
		local raySrc = player:getCameraPos()
		local rayDir = player:localToGlobal():withoutTranslation() * (-Vec3.e_z)
		local obj, dist = nil, nil
		for i, obj2 in ipairs(scene) do
			if isInstance(obj2, PhysicalObject) then
				local hit, dist2 = obj2:raycast(raySrc, rayDir)
				if hit and (dist == nil or dist2 < dist) then
					obj, dist = obj2, dist2
				end
			end
		end
		if obj and obj.grabbable and dist < 2 then
			holding = {
				obj = obj,
				viewModel = player:getCamera():globalToLocal() * obj:localToGlobal()
			}
		end
	end
end

function love.mousereleased(x, y, button)
	if not paused and button == 1 and holding ~= nil then
		holding.obj.orient[1] = 0
		holding.obj.orient[3] = 0
		holding = nil
	end
end

function love.keyreleased(key, code)
	if key == "escape" then
		paused = not paused
		love.mouse.setRelativeMode(not paused)
		if paused then
			love.mouse.setPosition(winW/2, winH/2)
		end
	end
end

local depthCanvasOpt = {
	format = "depth32f",
	readable = true
}

function love.resized(newWinW, newWinH)
	winW, winH = newWinW, newWinH
	resize3d(newWinW, newWinH)
end


local smallFont = love.graphics.newFont(20)
local bigFont = love.graphics.newFont(50)

function love.draw()
	render(scene, player:getCamera())
	if paused then
		love.graphics.setColor(0, 0, 0, 0.6)
		love.graphics.rectangle("fill", 0, 0, winW, winH)
		love.graphics.setColor(1, 1, 1)
		love.graphics.setFont(bigFont)
		love.graphics.printf("Paused", winW/4, winH*0.2, winW/2, "center")
	end
	
	love.graphics.setColor(1, 1, 1)
	love.graphics.setFont(smallFont)
	love.graphics.print(tostring(player.pos), 10, 10)
	love.graphics.print(tostring(player.orient), 10, 40)
	love.graphics.print(tostring(love.timer.getFPS()).." FPS", 10, 70)
end
