
shader = love.graphics.newShader([[
	varying vec3 normal;
	
	#ifdef VERTEX
		uniform mat4 proj_view;
		uniform mat4 model;
		
		attribute vec4 VertexNormal;
		
		vec4 position(mat4 transform_projection, vec4 vertex_position) {
			mat3 rot = mat3(model);
			normal = rot * vec3(VertexNormal);
			
			return proj_view * model * vertex_position;
		}
	#endif
	
	#ifdef PIXEL
		uniform float ambient_light;
		uniform float diffuse_light;
		uniform vec3 light_source_dir;
		
		uniform vec4 sky_color;
		uniform bool portal_surface;
		uniform Image surface_depth_canvas;
		
		vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
			if(portal_surface) return sky_color;
			
			vec4 texturecolor = Texel(texture, texture_coords);
			if(texturecolor.a == 0.0) discard;
			
			float diffuse = max(dot(light_source_dir, normal), 0)*diffuse_light;
			float light = ambient_light + diffuse;
			texturecolor.rgb *= min(light, 1);
			
			return texturecolor * color;
		}
	#endif
]])


invertShader = love.graphics.newShader([[
	extern Image bg_img;
	
	vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
		float alpha = Texel(texture, texture_coords).w * color.w;
		vec4 bgpixel = Texel(bg_img, vec2(screen_coords.x / love_ScreenSize.x, 1. - screen_coords.y / love_ScreenSize.y));
		return vec4(vec3(1., 1., 1.) - bgpixel.xyz, alpha);
	}
]])
